;==============================================================================
;	Solar OS - RealTime Operating System
;	Copyright (c) 2001-2007 Ontanu Bogdan Valentin, All rights reserved.
;	Bucuresti, Romania,Bd. Eroilor Sanitari nr 73B, suite 2
;	www.oby.ro, email : bogdanontanu@yahoo.com;
;===============================================================================


;-----------------------------------------
;	bootsector system loader 
;----------------------------------------


section "code" class_code

.code
	.use16

	nop

	org 	7C00h
	disp	7C00h

Boot_Start:
	jmp	boot02
	nop

;----------------------------------------
; note that i do not reserve the normal
; boot info used by FAT12 
; and MS filesystems
; so this floppy i not directly readable 
; under other OS'es
; aka not readable uned MS Win or DOS
; but any application that uses int13
; can read it fine
;----------------------------------------

sol_sign	db	"Sol boot v2007_05_12",13,10,0

;----------------------------------------
; data zone, variables we may need
;----------------------------------------
boot_drv	db	0	; drive number we where loaded from :)
cs_segment	dw	0

Trk_crt		db	0	;start load from this track
Sec_crt		db	2	;start load from this sector
Head_crt	db	0	;start load_from this head
Nr_Sectors	db	17	;how many sectors to read at once
Addr_crt	dw	07E0h	;start load at this segment
Read_try	db	0


;------------------------------------------------------------
; back to code...
; BIOS loads this first  512bytes sector at 0000:7C00h
;-------------------------------------------------------------
boot02:
	cli			; please no interupts
	mov	ax,cs
	mov	ds,ax
	mov	es,ax		; es same as ds

	mov	[boot_drv],dl	; save drive #

	;------------------------------------------------------
	; here we make our own stack
	; just under the A000h video memory
	;------------------------------------------------------
	mov	ax,09000h	; segment for stack
	mov	ss,ax
	mov	sp,0F800h	; offset for stack

	sti			; enable ints again

	;-------------------------------------
	; clear direction flag
	; repetitive string operations 
	; will increment SI/DI from now on
	;-------------------------------------
	cld
			
	;---------------------------------
	; print a short wellcome message
	; before anything else
	; so we know we are ok until here
	;---------------------------------
	mov	si,sol_sign
	call	Print_Msg 
	

;---------------------------------------------------------------------
; Load the Setup Code Sectors at linear 0x0000:7E00
; ie right after this boot code
; buffer is at es:bx ie 0x7c00:0200 and 0x7C00+0x0200=0x7E00
; 1.44M floppy 512 bytes/sector, 18 sectors/track, 2 sides, assumed
;---------------------------------------------------------------------
; This way of reading one sector at a time is slow esp on my laptop's
; external floopy; however it works relatively ok for desktop PCs
;
; todo: maybe attempt to read faster? multiple sectors at a time
;---------------------------------------------------------------------

load_setup:
	xor	cx,cx
	xor	ah,ah			; func.no.0 = reset FDC 
	mov	dl,[boot_drv]		; drive.no=0 ie disk a:
	int 	013h			; use bios call 

load_next:
	mov	ax,[Addr_crt]
	mov	es,ax			;segment for loading
	mov	bx,0			;offset of loading = 0

	mov	dl,[boot_drv]		
	mov	dh,[Head_crt]		
	mov	cl,[Sec_crt]		
	mov	ch,[Trk_crt]		

	mov	ah,2			; func.no.2=read sectors
	mov	al,1			; one sector at a time = slow  TODO:make it faster
				
	int	013h			; read sector via BIOS INT13 call
	jnc	ok_load_setup		; if ok - continue

	;-------------------------------------------
	; Ouch...here we have an error.... :(
	;-------------------------------------------
	Call	Print_fdd_error

	;-----------------------------------------
	; if error ... lets try it again ....
	;-----------------------------------------	
	jmp	load_setup

	;---------------------------------
	; ok load, at least one track ;)
	;---------------------------------
ok_load_setup:
	Call	Print_dot

	;-------------------------------------------------------------
	; goto next address
	; this will be in segment register 
	; so 20h is actually 0200h=512   aka x16
	;-------------------------------------------------------------
	mov	ax,[Addr_crt]
	add	ax,20h
	mov	[Addr_crt],ax

	;-----------------------------
	; next sector range 1..18
	;-----------------------------
	mov	al,[Sec_crt]
	inc	al
	mov	[Sec_crt],al
	cmp	al,19
	jc	load_next

	mov	al,1
	mov	[Sec_crt],al

	;---------------------------------
	; next head hardware range: 0..1
	;---------------------------------
	mov	al,[Head_crt]
	inc	al
	mov	[Head_crt],al
	cmp	al,2
	jc	load_next

	;----------------------
	; reset head
	;----------------------
	mov	al,0
	mov	[Head_crt],al

	;---------------------------------------
	; next track my range 0..2,3,4
	; this will grow as the system grows?
	;---------------------------------------

	;------------------------------------------
	; this size has to be greater than sizeof
	; os8.bin = boot+system16+system32 
	;------------------------------------------
	;  4 tracks =  4x2x18x512 =  73.728 bytes
	;  5 tracks =  5x2x18x512 =  92.160 bytes 
	;  6 tracks =  6x2x18x512 = 110.592 bytes
	;  7 tracks =  7x2x18x512 = 129.024 bytes
	;  8 tracks =  8x2x18x512 = 147.456 bytes
	;  9 tracks =  9x2x18x512 = 165.888 bytes
	;
	; 12 tracks = 12x2x18x512 = 221.184 bytes
	;
	; 16 tracks = 16x2x18x512 = 294.912 bytes
	; 17 tracks = 17x2x18x512 = 313.344 bytes
	; 18 tracks = 18x2x18x512 = 331.776 bytes
	; 19 tracks = 19x2x18x512 = 350.208 bytes
	;
	; 26 tracks = 26x2x18x512 = 479,232 bytes
	;
	; 30 tracks = 30x2x18x512 = 552.960 bytes
	;
	; 32 tracks = 32x2x18x512 = 589.824 bytes
	; 33 tracks = 33x2x18x512 = 608.256 bytes
	; 34 tracks = 34x2x18x512 = 626.688 bytes
	;------------------------------------------
	mov	al,[Trk_crt]
	inc	al
	mov	[Trk_crt],al

	cmp	al,32
	jc	load_next

	;-----------------------------
	; ALL load is done
	; print a happy message...
	;------------------------------
	mov	si,msg_002	
	call	Print_Msg 		 

	;-------------------------------------------------------
	; nothig more to do here ... eh such is life
	; bye-bye "First Sector" for now;
	; meet me in Gamma Quadrant sometime in the future...
	;-------------------------------------------------------
	;	jmp	$

	;-------------------------------------------------------------------
	; QUICK HACK FOR A -FAR- JMP INSTRUCTION
	;-------------------------------------------------------------------
		DB 0EAh			; jump to sector 02 start
		DW 08000h		; offset 
		DW 00000h		; segment 


;***************************************
; utility routines for the bootsector
;***************************************

;====================================================
; First a message printing routine
; using bios call to output ASCII letters to Screen
; until the last char=0 when it stops and returns
; aka C style sz_string
; IN: SI sz_string pointer
;===================================================
Print_Msg:
	lodsb		; al=[ds:si], si=si+1 ;)
	or	al,al
	jz	ok_msg1

	;------------------------------------------------------
	; BIOS int 0x10 function 0x0E = write char to console
	;------------------------------------------------------
	push	si
	mov	ah,0Eh	; ah=0x0E al=ascii code of char to output
	int	10h
	pop	si
	jmp	Print_Msg
ok_msg1:
	ret

Wait_Key:
	;-----------------------------
	; wait for a key
	;-----------------------------	
	mov	ah,0
	int	16h
ret

Print_Space:	
	mov	ah,0Eh
	mov	al,20h	
	int	10h
	ret

;====================================
; Print newline to Screen
; CR and LF are actually a newline
;===================================
Print_CRLF:	
	mov	ax,0E0Dh	; al=CR=0x0D, ah=0x0E=print char fn
	int	10h
	mov	ax,0E0Ah	; al=LF=0x0A
	int	10h
	ret

Print_dot:	
	mov	ah,0Eh
	mov	al,"#"	
	int	10h
	ret

;=======================================================
; print 16 bit value from DX in hexa format on Screen
; DX register has the value to print
;======================================================
Print_Hex16:
	mov   cx, 4      	; 4 hex digits	
				
PrintDigit:
	rol   dx, 4      	; rotate so that lowest 4 bits are used
	mov   ax, 0E0Fh		; ah = request, al = mask for nybble
	and   al, dl

	add   al, 90h		; convert al to ascii hex (four instructions)
	daa			
	adc   al, 40h		
	daa			

	int   10h		; call BIOS to print char
	dec	cx
	jnz	PrintDigit
	ret

;======================================
; Show Err_code:Trk:Head:Sector:Address
;======================================
Print_fdd_error:
	push	ax		;save error code
	mov	esi,msg_003
	Call	Print_Msg
	pop	ax		; retrive error code

	mov	dx,ax
	call	Print_Hex16	; print error code

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Trk_crt]
	mov	dh,0
	call	Print_Hex16	; print track

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Head_crt]
	mov	dh,0
	call	Print_Hex16	; print head

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Sec_crt]
	mov	dh,0
	call	Print_Hex16	; print sector

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dx,[Addr_crt]
	call	Print_Hex16	; print buff address
;---------------------------
; wait a key to pause
;---------------------------
	mov	ah,0
	int	16h
ret

/*

;======================================
; Show Err_code:Trk:Head:Sector:Address
;======================================
Print_fdd_info:
	mov	esi,offset msg_004
	Call	Print_Msg


	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Trk_crt]
	mov	dh,0
	call	Print_Hex16	; print track

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Head_crt]
	mov	dh,0
	call	Print_Hex16	; print head

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dl,[Sec_crt]
	mov	dh,0
	call	Print_Hex16	; print sector

	mov	ah,0Eh	; print separator
	mov	al,":"
	int	10h
	mov	dx,[Addr_crt]
	call	Print_Hex16	; print buff address
;---------------------------
; wait a key to pause
;---------------------------
	mov	ah,0
	int	16h
ret

*/

;========================================================
; some messages i use in boot sector
;========================================================
msg_002:	db	13,10,">Setup Code Loaded Ok! :) ",13,10,0
msg_003:	db	13,10
		db	">Fdd RD Err! (code,trk,head,sect,addr)"
		db	13,10,0
msg_004		db	13,10,"Fdd Read:",0

;========================================
; boot disk signature is required 
; at end of Sector 1 Track 0
; or else BIOS will not execute this
; as a valid boot sector
;
; warn:take care that boot sector code 
; is not greater than 510 bytes!
;========================================
BootSignature:
	ORG	Boot_Start+510
	db	055h,0AAh 	



;*******************************************
; HERE sector 01  Physically ends
;*******************************************



