;============================================
; Solar OS sample
; to be used with Solar Assembler
;--------------------------------------------
; This sample shows how to use a 
; ListBox control in SOL
;============================================


;----------------------------
; import from this DLL
;----------------------------
import_dll ..\sol_os_kernel.dll

;----------------------------
; the following functions
;----------------------------
	import_func	API_Window_Create
	import_func	API_Window_Set_Caption
	import_func	API_Window_Set_Callback
	import_func	API_Window_Set_Min_Size

	import_func	API_Text_Draw
	import_func	Str_Printf CDECL

	import_func	API_List_Box_Setup
	import_func	API_List_Box_Str_Del
	import_func	API_List_Box_Str_Add
	import_func	API_List_Box_Clear_All

	import_func	API_List_Box_Item_Addr_Get
	import_func	API_List_Box_Item_Id_Get
	import_func	API_List_Box_Item_Sel_Get


;----------------------
; define sections
;----------------------
section "code"		class_code
section "data" 		class_data
section	"idata"		class_imports
section "reloc" 	class_relocs


;---------------------
;define entry point
;---------------------
.entry App_Init


;------------------------------
; includes
;------------------------------
include		..\sol32_inc.asm


.data
	main_handle		dd	0
	main_caption		db	"Sol List Box Sample",0


	list_handle		dd	0
	list_sel_item		dd	-1


	sel_head		db	"Selected:",0
	sel_fmt1		db	" pos: %u",0
	sel_fmt2		db	" id : %x",0
	sel_fmt3		db	" str: %s",0

	item_fmt		db	"Item:%u",0

	tmp_buff1		rb	128
.code


App_Init:
	
	;----------------------------------
	; create main application window
	;----------------------------------	
	mov	eax,FLAG_WND_ALPHA+FLAG_WND_MINI
	invoke	API_Window_Create,0,128,128,320,200,eax,WND_TYPE_TOP
	mov	[main_handle],eax

	;-----------------
	; set caption
	;-----------------
	invoke	API_Window_Set_Caption, eax, main_caption

	;-----------------
	; set callback
	;-----------------
	invoke	API_Window_Set_Callback,[main_handle],OS_CALL_AFTER,Main_Callback

	;-------------------------------------
	; set minimum resize values
	;-------------------------------------
	invoke	API_Window_Set_Min_Size,[main_handle],200,108

	;------------------------------
	; create list box control
	;------------------------------
	mov	eax,FLAG_WND_NO_MOVE+FLAG_WND_KEEP_DY+FLAG_WND_NO_RESIZE
	invoke	API_Window_Create,[main_handle],8,28,132,159,eax,WND_TYPE_LIST_BOX
	mov	[list_handle],eax

	;------------------------------------
	; add some dummy items to list box
	;------------------------------------
	invoke	List_Box_Init,eax

	;------------------------------------
	; not used in this sample
	; since we do not need to handle 
	; list box events
	;------------------------------------
	; invoke	API_Window_Set_Callback,[list_handle],OS_CALL_AFTER,List_Box_Callback

	ret



PROC List_Box_Init stdcall
	USES	edx,ecx
	ARG	list_handle

	;------------------------------------------------
	; Brief:
	; - this will init the listbox to have 7 items
	;------------------------------------------------
	mov	ecx,7

loop_items:
	;-----------------------
	; prepare item string
	;-----------------------
	invoke	Str_Printf,tmp_buff1,item_fmt,ecx

	;--------------------------------
	; for fun make ID = ecx + 1
	; this should be app defined
	;--------------------------------
	mov	edx,ecx
	inc	edx

	;-----------------------
	; add item to listbox
	;-----------------------
	invoke	API_List_Box_Str_Add,[list_handle],tmp_buff1,edx

	;---------------------
	; next item counter
	;---------------------
	dec	ecx
	jnz	loop_items

	ret
ENDP



PROC Main_Callback STDCALL
	USES	ebx,edx,esi,edi
	ARG	wnd_handle,wnd_action,wnd_param1,wnd_param2
	LOCAL	item_ptr	
	;------------------------------------
	; switch on message type
	;------------------------------------
	mov	eax,[wnd_action]

	.IF eax == ACT_PAINT_CHILD
	
		;-----------------------
		; paint text header
		;-----------------------
		invoke	API_Text_Draw,[wnd_handle],160,32,sel_head,0ff1f1fh

		;---------------------------------
		; get list box selected item info
		;---------------------------------
		invoke	API_List_Box_Item_Sel_Get,[list_handle]
		mov	[item_ptr],esi


		;-------------------------------
		; show selected item position
		;-------------------------------
		invoke	Str_Printf,tmp_buff1,sel_fmt1,eax
		invoke	API_Text_Draw,[wnd_handle],164,48,tmp_buff1,0ffff1fh

		;-------------------------------
		; show selected item ID
		;-------------------------------		
		mov	eax,[esi+LIST_BOX_ITEM_HEAD.item_id]
		invoke	Str_Printf,tmp_buff1,sel_fmt2,eax
		invoke	API_Text_Draw,[wnd_handle],164,64,tmp_buff1,0ffff1fh

		;-------------------------------
		; show selected item string
		;-------------------------------		
		add	esi,LIST_BOX_ITEM_HEAD.item_string
		invoke	Str_Printf,tmp_buff1,sel_fmt3,esi
		invoke	API_Text_Draw,[wnd_handle],164,80,tmp_buff1,0ffff1fh

	.ENDIF

	xor	eax,eax
	ret
ENDP



;------------------------------------------------------------
; this could be used to make things more efficient
; aka perform actions on events rather than on paint above
; however it will be slightly more complicated
;------------------------------------------------------------
PROC List_Box_Callback STDCALL
	USES	ebx,edx,esi,edi
	ARG	wnd_handle,wnd_action,wnd_param1,wnd_param2

	;------------------------------------
	; switch on message type
	;------------------------------------
	mov	eax,[wnd_action]	
	.IF eax == ACT_LIST_SEL_ITEM
		;--------------------------------------
		; this event will happend when 
		; you select a new item in listbox
		; wnd_param1 = selected item pos
		;--------------------------------------
		mov	eax,[wnd_param1]		
								
	.ENDIF

	xor	eax,eax
	ret
ENDP




;******************************
;  END OF PROGRAM TEXT
;******************************


